
class HandlerNotFoundError(Exception):
    pass


class MessageSingleHandlerBroken(Exception):
    pass


class ExtractorMatchNotFound(Exception):
    pass
    

class MalformedMessageError(Exception):
    pass
